﻿using System;
using System.IO;
using System.Reflection;
using UnityModManagerNet;
using HarmonyLib;
using UnityEngine;

namespace doot;

[EnableReloading]
static class Main
{
	private static bool enabled;
	private static UnityModManager.ModEntry myModEntry;
	private static Harmony myHarmony;

	public static AudioClip DootClip;

	//================================================================

	private static bool Load(UnityModManager.ModEntry modEntry)
	{
		try
		{
			myModEntry = modEntry;
			
			modEntry.OnToggle = OnToggle;
			modEntry.OnUnload = Unload;
			
			DootClip = AssetBundle.LoadFromFile(Path.Combine(myModEntry.Path, "dootbundle"))
				.LoadAsset<AudioClip>("Assets/doot.mp3");

			myHarmony = new Harmony(myModEntry.Info.Id);
			myHarmony.PatchAll(Assembly.GetExecutingAssembly());
		}
		catch (Exception ex)
		{
			myModEntry.Logger.LogException($"Failed to load {myModEntry.Info.DisplayName}:", ex);
			Unload(modEntry);
			return false;
		}
		
		modEntry.Logger.Log("loaded");

		return true;
	}

	private static bool Unload(UnityModManager.ModEntry modEntry)
	{
		myHarmony?.UnpatchAll(myModEntry.Info.Id);
		AssetBundle.UnloadAllAssetBundles(false);
		return true;
	}

	private static bool OnToggle(UnityModManager.ModEntry modEntry, bool value) 
	{
		enabled = value;
		string msg = enabled ? "hello!" : "goodbye!";
		modEntry.Logger.Log(msg);

		return true;
	}

	// Logger Commands
	public static void Log(string message)
	{
		myModEntry.Logger.Log(message);
	}

	public static void Warning(string message)
	{
		myModEntry.Logger.Warning(message);
	}

	public static void Error(string message)
	{
		myModEntry.Logger.Error(message);
	}
}